/* Copyright 2009, UCAR/Unidata and OPeNDAP, Inc.
   See the COPYRIGHT file for more information. */

#ifndef OCINTERNAL_H
#define OCINTERNAL_H

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <stdlib.h>
#define CURL_DISABLE_TYPECHECK 1
#include <curl/curl.h>

#include <assert.h>
#include <string.h>
#include <stdarg.h>

/*#include <rpc/types.h>*/

#ifdef HAVE_RPC_TYPES_H
#include <rpc/types.h>
#endif

/*#include <rpc/xdr.h>*/

#ifdef HAVE_RPC_XDR_H
#include <rpc/xdr.h>
#else
#include <xdr.h>
#endif

#include "oclist.h"
#include "ocbytes.h"

#define OCCACHEPOS
#ifdef OCCACHEPOS
extern void ocxdrstdio_create(XDR*,FILE*,enum xdr_op);
#else
#define ocxdrstdio_create(xdrs,file,op) xdrstdio_create(xdrs,file,op)
#endif

#include "config.h"

#include "oc.h"
#include "ocdatatypes.h"
#include "constraints.h"
#include "ocnode.h"
#include "dapurl.h"
#include "ocutil.h"
#include "oclog.h"
#include "ocdata.h"

#define nulldup(s) (s==NULL?NULL:strdup(s))
#define nullstring(s) (s==NULL?"(null)":s)

#define PATHSEPARATOR "."

/* Compile when |datadds| < 1gig */
#define OCCOMPILELIMIT (0x40000000)

/*! Specifies the OCstate. */
typedef struct OCstate
{
    CURL* curl; /* curl handle*/
    OClist* trees; /* list<OCnode*> ; all root objects */
    DAPURL url; /* base URL */
    OCbytes* packet; /* shared by all trees during construction */
    /* OCContent information */
    struct OCcontent* contentlist;
    struct {/* Hold info for an error return from server */
	char* code;
	char* message;
	long  httpcode;
    } error;
} OCstate;

/*! Specifies all the info about a particular DAP tree
    i.e. DAS, DDS, or DATADDS as obtained from a fetch response
    This is associated with the root object.
*/
typedef struct OCtree
{
    OCdxd  dxdclass;
    char* constraint;
    char* text;
    struct OCnode* root; /* cross link */
    struct OCstate* state; /* cross link */
    OClist* nodes; /* all nodes in tree*/
    /* when dxdclass == OCDATADDS */
    struct {
        unsigned long bod;
        char* filename;
        FILE* file;
        unsigned long filesize;
        XDR* xdrs;
        struct OCmemdata* memdata; /* !NULL iff compiled */
    } data;
} OCtree;

/*
WARNING: this data structures uses the standard C trick of
casting a long memory chunk to be an instance of this
object, which means that the data part may actually be
longer than 8 chars.
*/
typedef struct OCmemdata {
    OCtype octype; /* Actually instance of OCtype, but guaranteed to be |long| */
    OCtype etype; /* Actually instance of OCtype, but guaranteed to be |long| */
    OCmode mode; /* Actually instance of OCmode, but guaranteed to be |long| */
    unsigned long count; /* count*octypesize(datatype) == |data| */
    union {
        struct OCmemdata* mdata[2];
        unsigned int* idata[2];
        char data[8]; /* Actually prefix of the data; want to start on longlong boundary */
	char* sdata;
    } data;
} OCmemdata;


/* (Almost) All shared procedure definitions are kept here
   except for: ocdebug.h ocutil.h
   The true external interfac is defined in oc.h
*/

/* Location: ocnode.c */
/*
extern OCnode* makepseudodimension(size_t size, OCnode* array, int index);
*/
extern OCnode* makeocnode(char* name, OCtype ptype, OCnode* root);
extern void collectpathtonode(OCnode* node, OClist* path);
extern void computeocfullnames(OCnode* root);
extern void computeocsemantics(OClist*);
extern void addattribute(OCattribute* attr, OCnode* parent);
extern OCattribute* makeattribute(char* name, OCtype ptype, OClist* values);
extern size_t ocsetsize(OCnode* node);
extern OCerror occorrelate(OCnode*,OCnode*);

/* Location: dapparselex.c*/
extern int dapdebug;
extern OCerror DAPparse(OCstate*, struct OCtree*, char*);
extern char* dimnameanon(char* basename, unsigned int index);

/* Location: ceparselex.c*/
extern int cedebug;
extern OClist* CEparse(OCstate*,char* input);

/* Location: occompile.c*/
extern int occompile(OCstate* state, OCnode* root);
extern void freeocmemdata(OCmemdata* md);
extern void octempclear(OCstate* state);

/* Location: ocinternal.c*/
extern OCerror ocopen(OCstate** statep, const char* url);
extern void occlose(OCstate* state);

extern OCerror ocfetch(OCstate*, const char*, OCdxd, OCnode**);

/* Location: ocnode.c */
extern void ocfreetree(OCtree* tree);
extern void ocfreeroot(OCnode* root);
extern void ocfreenodes(OClist*);

extern void ocddsclear(struct OCstate*);
extern void ocdasclear(struct OCstate*);
extern void ocdataddsclear(struct OCstate*);


/* Merge DAS with DDS or DATADDS*/
extern int ocddsdasmerge(struct OCstate*, OCnode* das, OCnode* dds);

#endif /*COMMON_H*/
